#
# Interface to the mkisofs and cdrecord Linux commands.
#
# Copyright (C) 1999 Steve Ratcliffe
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

import string
import command
import cdbak

##
def mkisofs(conf, section, filelist):
	"""
	Take a list of options and a list of path names and run the mkisofs
	command to create the file system.
	"""

	argv = []

	argv.append(get_value(conf, section, 'mkisofs', 'mkisofs'))

	# Set the output file
	argv.append('-o')
	argv.append(conf.get(section, 'output'))

	if get_bool(conf, section, 'allfiles'):
		argv.append('-a')

	val = get_value(conf, section, 'application_id')
	if val:
		argv.append('-A')
		argv.append(val)

	val = get_bool(conf, section, 'rockridge')
	val2 = get_bool(conf, section, 'rockridge_dist')
	if val:
		if val2:
			argv.append('-r')
		else:
			argv.append('-R')

	# Now follows strings of file system directories and the corresponding
	# directories on the CD.  The CD place comes first.  The trailing
	# slashes are significant and mean that it will be a directory
	#
	fileargv = []
	exclude_argv = []
	for (fs, cd, excludelist) in filelist:
		# The default is the root directory on the CD
		if cd == '/':
			fileargv.append(fs)
		else:
			fileargv.append(cd + '=' + fs)

		# Add -m options for each excluded path
		for e in excludelist:
			exclude_argv.append('-m')
			exclude_argv.append(e)

	argv = argv + exclude_argv + fileargv
	c = command.Command(argv)

	return c

# Build up a cdrecord command based on the specified options.
def cdrecord(conf, section):
	argv = []

	val = get_value(conf, section, 'cdrecord', 'cdrecord')
	argv.append(val)

	argv.append('-v')
	#argv.append('-multi')

	val = get_value(conf, section, 'device')
	if val:
		argv.append('dev=%s' % val)
	else:
		raise cdbak.error, "No 'device' parameter specified"

	val = get_value(conf, section, 'speed', 1)
	if val:
		argv.append('speed=%s' % val)

	val = get_value(conf, section, 'output')
	if val is None:
		raise cdbak.error, "No 'output' parameter specified"

	if val == '/dev/null':
		return command.Command(string.split(
			"echo OK, Just testing! (output of /dev/null specified)"))
	
	argv.append(val)

	#print "cdrecord CMD:", argv
	c = command.Command(argv)

	return c

#### Helper utilities
##
def get_value(conf, section, name, d=None):
	"""
	Return the value of parameter name from conf.  If not specified then
	the default is returned.

	If no default given, it is the name of the variable.
	"""

	try:
		val = conf.get(section, name)
	except:
		val = d

	return val


def get_bool(conf, section, name):
	try: val = conf.get(section, name)
	except: return 0

	if string.lower(val[:1]) == 'y':
		return 1
	else:
		return 0
