#!/usr/bin/python -O
#
# the gui for cdbak
#
# Copyright (C) 1999 Steve Ratcliffe
#
#  This program is free software; you can redistribute it and/or modify
#  it under the terms of the GNU General Public License version 2 as
#  published by the Free Software Foundation.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#

from gtk import *
import command

# Contants to show what has been done
PROFILE_SELECTED = 1

mkfs_help_text= """
Select the backup
profile from the
drop down list.
"""

write_help_text="""
If all seemed to go well
with preparing the filesystem,
ensure that there is a CD in
the drive.
"""

finish_help_text ="""
CD Backup is complete
"""

# String that shows that no choice was made
no_selection = '--select--'

# Quit the whole display
def quit(win, event=None):
	command.killall()
	mainquit()
	return 0

# Add widget to container and show it
def addbox(box, w, expand=TRUE, fill=TRUE, padding=0):
	box.pack_start(w, expand, fill, padding)
	w.show()

class CdbakGui:

	def __init__(self, FileList, cback):
		self.FileList = FileList
		self.EventCallback = cback

		self.Page = 0
		self.NPages = 0
		self.PageInfo = []
		self.AllowChange = 0

		self.Labels = {}
		self.NextFinishText = ['']


	# Draw everything on the main screen
	#
	def init_screen(self):
		win = GtkWindow(WINDOW_TOPLEVEL)
		win.set_default_size(screen_width()*60/100, screen_height()*80/100)
		win.set_title("CD backup")

		#
		# The application is made from a series of panels placed
		# on top of each other.
		#
		mainvbox = GtkVBox(0)
		win.add(mainvbox)
		mainvbox.show()
		win.connect("delete_event", quit)

		w = self.topbox()
		w.set_usize(0, 60)
		addbox(mainvbox, w, expand=0)

		w = self.userinput()
		w.set_usize(100, 200)
		addbox(mainvbox, w, expand=0)

		w = self.buttonrow()
		addbox(mainvbox, w, expand=0)

		w = self.status()
		addbox(mainvbox, w, fill=1)
		
		self.update_pages()	
		win.show()

		# Make the dialogue associated with the main window
		# and positioned conveniently for the mouse.
		#
		self.EventCallback.okDialog.set_position(WIN_POS_MOUSE)
		self.EventCallback.okDialog.set_transient_for(win)

	# The top of the Screen, this is basically a title and
	# welcome panel
	#
	def topbox(self):
		frame = GtkFrame("CD backup")
		welcome = GtkLabel("Welcome to CDBAK")
		frame.add(welcome)
		welcome.show()
		return frame


	#
	# This is the main user interaction window.  There is a sequence of
	# screens that you can step through.
	#
	def userinput(self):
		nb = GtkNotebook()
		nb.set_show_tabs(0)

		# Save the book
		self.Userbook = nb

		# Add in the pages
		w = self.mkfs_page()
		self.addpage(w, 'make')

		w = self.progress_page('make')
		self.addpage(w, 'make_progress')

		w = self.write_page()
		self.addpage(w, 'write')

		w = self.progress_page('write')
		self.addpage(w, 'write_progress')

		w = self.finish_page()
		self.addpage(w, 'finished')

		return nb

	def buttonrow(self):
		bbox = GtkHButtonBox()
		bbox.set_border_width(6)
		bbox.set_spacing(5)
		bprev = GtkButton("<< Prev")
		addbox(bbox, bprev)

		bnext = GtkButton()
		addbox(bbox, bnext)

		bquit = GtkButton("Quit")
		addbox(bbox, bquit)

		# Save the ones that we need to access elsewhere
		self.Next = bnext
		self.Prev = bprev

		bquit.connect("clicked", quit, 'hello')
		bnext.connect("clicked", self.onNextPrev, 1)
		bprev.connect("clicked", self.onNextPrev, -1)

		# The Next button can also be labeled 'Finish' so we need
		# to save the Label so we can change the text on it
		nlab = GtkLabel("Next >>")
		self.Labels['Next'] = nlab
		self.Next.add(nlab)	# Add to the button
		nlab.show()

		hbox = GtkHBox()
		hbox.pack_start(bbox, expand=0)
		bbox.show()
		return hbox

	#
	# The next routines construct the controls that appear in the
	# middle of the application.  The next and prev buttons
	# step among them.
	#

	# Select the backup type, this consists of a help label, a
	# combo box with the possible options and the list of selected files.
	#
	def mkfs_page(self):
		frame = GtkFrame("Make filesystem")

		tab = GtkTable(2, 2, homogeneous=0)

		fslabel = GtkLabel(mkfs_help_text % {'action':self.NextFinishText[0]})
		tab.attach(fslabel, 0, 1, 0, 1, xoptions=0, xpadding=5)
		fslabel.show()

		cs, filetab = self._combo_select()
		tab.attach(cs, 0, 1, 1, 2, xoptions=0, xpadding=5)
		tab.attach(filetab, 1, 2, 0, 2, xoptions=EXPAND|FILL)
		cs.show()
		filetab.show()

		frame.add(tab)
		tab.show()

		return frame


	# Shows the progress of the current action.
	#
	def progress_page(self, type):
		frame = GtkFrame()

		box = GtkVBox()
		frame.add(box)
		box.show()

		lab = GtkLabel("Please wait while the backup is prepared")
		addbox(box, lab, padding=4)

		prog = GtkProgressBar()
		addbox(box, prog, expand=0)

		if type == 'make':
			description = "Preparing backup"
			self.progressPrepare = prog
		else:
			description = "Writing CD"
			self.progressWrite = prog

		frame.set_label(description)
		prog.set_show_text(1)
		prog.set_format_string(description)

		# Create a big Cancel button
		bbox = GtkHButtonBox()
		bcancel = GtkButton("Cancel")
		bcancel.set_border_width(10)
		bcancel.connect("clicked", self.onCancel, type)
		addbox(bbox, bcancel)
		addbox(box, bbox, fill=0)

		prog.update(0.0)

		return frame


	# The screen to set up writing the CD
	#
	def write_page(self):
		lab = GtkLabel(write_help_text)
		return lab
		
	# The page shown when all is finished
	#
	def finish_page(self):
		lab = GtkLabel(finish_help_text)
		return lab

	#
	# Status box at the bottom that contains output lines
	# from commands that are run.
	#
	def status(self):
		frame = GtkFrame("Command output")
		text = GtkText()
		sw = GtkScrolledWindow()
		sw.set_policy(POLICY_NEVER, POLICY_ALWAYS)

		frame.add(sw)
		sw.show()

		sw.add(text)
		text.show()

		self.StatusText = text
		return frame


	# Return a combo box of the filesystem choices an a list box showing
	# the currently selected profile.
	#
	def _combo_select(self):

		combo = GtkCombo()

		strs = [no_selection]
		for (name, l) in self.FileList:
			strs = strs + [name]
		combo.set_popdown_strings(strs)
		combo.entry.set_editable(0)	# Disable editing

		"""
		tab = GtkTable(3, 2)
		head1 = GtkLabel("From")
		head2 = GtkLabel("On CD")
		tab.attach(head1, 0, 1, 0, 1, yoptions=0, ypadding=4)
		tab.attach(head2, 2, 3, 0, 1, yoptions=0, ypadding=4)
		head1.show()
		head2.show()
#
		frombox = GtkList()
		tobox = GtkList()


		frombox.show()
		tobox.show()

		tab.attach(frombox, 0, 1, 1, 2)
		tab.attach(tobox, 2, 3, 1, 2)

		sep = GtkVSeparator()
		tab.attach(sep, 1, 2, 0, 2, xoptions=0)
		sep.show()
		"""

		cl = GtkCList(2, ['Path', 'On CD'])
		cl.column_titles_passive()
		cl.set_usize(100, 100)
		cl.set_column_min_width(0, 200)
		cl.set_column_min_width(1, 200)

		combo.entry.connect("changed", self.set_file_list, cl)
		# Put the lists in a frame as it looks better
		#frame = GtkFrame()
		#frame.add(cl)
		#cl.show()
		sw = GtkScrolledWindow()
		sw.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		sw.add(cl)
		cl.show()

		return combo, sw

	# This fills in the list of directories that are selected for
	# backing up.
	def set_file_list(self, entry, cl):
		sel = entry.get_text()
		cl.clear()

		if sel != no_selection:
			for (name, selected_file_list) in self.FileList:
				if sel == name:
					sellist = selected_file_list
					for (f, t, exceptlist) in sellist:
						cl.append((f, t))
						for e in exceptlist:
							cl.append(('  '+e, '  <excluded>'))
					break
			self.EventCallback.profileSelect(sel, sellist)

	### General routines.

	# Update everything to do with the pages interface
	#
	def update_pages(self):
		if self.AllowChange == 0:
			self.Next.set_sensitive(0)
			self.Prev.set_sensitive(0)
		else:
			if self.Page == 0:
				self.Prev.set_sensitive(0)
			else:
				self.Prev.set_sensitive(1)
			if self.Page >= self.NPages-1:
				self.Next.set_sensitive(0)
			else:
				self.Next.set_sensitive(1)

		self.NextFinishText[0] = "Next >>"
		self.Labels['Next'].set_text(self.NextFinishText[0])
		self.Userbook.set_page(self.Page)

	### Callback routines that work within the gui

	def onNextPrev(self, button, inc):
		# Tell Calling application, gets the page name that we are on
		self.AllowChange = 0	# Wait for app to turn them back on
		self.EventCallback.onNextPrev(self.PageInfo[self.Page], inc)

		self.update_pages()

	def onCancel(self, button, data):
		self.EventCallback.onCancel(type)
		self.AllowChange = 1

	def addpage(self, p, name):
		self.Userbook.add(p)
		self.PageInfo = self.PageInfo + [(name, self.NPages)]
		self.NPages = self.NPages + 1
		p.show()

	# Allow changes via the prev and next buttons
	def allow_step(self, yes=1):
		if yes:
			self.AllowChange = 1
		else:
			self.AllowChange = 0
		self.update_pages()

	def set_page(self, page):
		""" Set the page of the user interface """

		if type(page) == type(0):
			self.Page = page
		else:
			for (name, num) in self.PageInfo:
				if name == page:
					self.Page = num

		if self.Page < 0: self.Page = 0
		if self.Page >= self.NPages: self.Page = self.NPages-1
		self.update_pages()

class OKdialog(GtkDialog):
	def __init__(self):
		GtkDialog.__init__(self)

		self.connect("delete_event", self.take_down)
		self.set_usize(400, 200)

		lab = GtkLabel("Error occured")		# This should always be re-set
		lab2 = GtkLabel("Click OK to continue")

		# Add the labels
		self.vbox.add(lab)
		self.vbox.add(lab2)
		lab.show()
		lab2.show()
		
		bbox = GtkHButtonBox()
		ok = GtkButton("OK")
		ok.set_flags(CAN_DEFAULT)
		ok.connect("clicked", self.onClick)

		bbox.add(ok)
		ok.show()

		# Add to the action area part of the dialog
		self.action_area.add(bbox)
		bbox.show()

		ok.grab_default()
		self.OKbutton = ok
		self.messageLabel = lab

	# Pop up the dialog and show the message.
	def show_message(self, msg, func=None, arg=None):
		self._set_action(func, arg)
		self.messageLabel.set_text(msg)
		self.show()
		grab_add(self)

	# Set the action to take when the OK button is clicked
	def _set_action(self, func, arg):
		self.okFunc = func
		self.okFuncArg = arg

	def onClick(self, w):
		grab_remove(self)
		self.hide()
		if self.okFunc:
			self.okFunc(self.okFuncArg)

	def take_down(self, w, foo):
		self.destroy()

########### Test stuff
# Test list
tList = [
	('hello', [('/var', 'var'), ('/home/steve', 'steve')]),
	('world', [('/var', 'var'), ('/home/steve', 'steve'), ('/usr/src', 'src')]),
	('another', [('/usr/local', 'local'), ('/var', 'var'), ('/home/steve', 'steve'), ('/usr/src', 'src')]),
	('one', [('/home/soffice', 'soffice'), ('/usr/local', 'local'), ('/var', 'var'), ('/home/steve', 'steve'), ('/usr/src', 'src')]),
	]

if __name__ == '__main__':
	# Just for testing
	class cback:
		def onNext(self, (name, obj)):
			self.FrontEnd.allow_step()

		def onCancel(self, what):
			self.FrontEnd.allow_step()

		def profileSelect(self, name):
			if name[:2] != '--':
				self.FrontEnd.allow_step()

		def set_frontend(self, w):
			self.FrontEnd = w

	cb = cback()
	w = CdbakGui(tList, cb)
	cb.set_frontend(w)
	w.init_screen()
	mainloop()
